"use client";

import type React from "react";
import { useEffect, useRef, useState } from "react";

const localStorage = typeof window !== "undefined" ? window.localStorage : null;

/**
 * A custom React hook that allows you to store and retrieve data in the browser's localStorage.
 *
 * @param {string} key - The key under which the data should be stored in localStorage.
 * @param {T} initial - The initial value for the stored data.
 * @returns {[T, React.Dispatch<React.SetStateAction<T>>]} - An array containing the stored value and a function to update the stored value.
 */
export function useLocalStorage<T>(
	key: string,
	initial: T,
): [T, React.Dispatch<React.SetStateAction<T>>] {
	const readValue = () => {
		const item = localStorage?.getItem(key);
		if (item) {
			try {
				return JSON.parse(item);
			} catch (error) {
				console.warn(`Error reading localStorage key “${key}”:`, error);
			}
		}
		return initial;
	};

	const [storedValue, setStoredValue] = useState<T>(readValue);

	useEffect(() => {
		try {
			localStorage?.setItem(key, JSON.stringify(storedValue));
		} catch (error) {
			console.warn(`Error setting localStorage key “${key}”:`, error);
		}
	}, [key, storedValue]);
	return [storedValue, setStoredValue];
}

/**
 * Custom hook that provides an encoded version of a state value and a function to update that value.
 *
 * @template T - The type of the state value.
 * @param {string} key - The key under which the state value is stored in the local storage.
 * @param {T} initialValue - The initial value of the state.
 * @return {[T, React.Dispatch<React.SetStateAction<T>>]} - A tuple containing the encoded state value and the function to update it.
 */
export function useEncodedLocalStorage<T>(key: string, initialValue: T): readonly [T, React.Dispatch<React.SetStateAction<T>>] {
	const [encodedState, setEncodedState] = useState<T>(() => getFromLocalStorage(key, initialValue));

	useEffect(() => {
		saveToLocalStorage(key, encodedState);
	}, [encodedState, key]);

	return [encodedState, setEncodedState] as const;
}

/**
 * Retrieves a value from the localStorage with the given key.
 *
 * @param {string} key - The key to retrieve the value from.
 * @param {T} initialValue - The initial value to be returned if the value does not exist in the localStorage.
 * @returns {T} - The retrieved value from the localStorage or the initial value if it does not exist.
 */
function getFromLocalStorage<T>(key: string, initialValue: T): T {
	try {
		const localStorageValue = localStorage?.getItem(key);
		if (localStorageValue) {
			return decodeFromBase64(localStorageValue);
		}
		return initialValue;
	} catch {
		return initialValue;
	}
}

/**
 * Save a value to the local storage.
 *
 * @param {string} key - The key to store the value.
 * @param {*} value - The value to be stored.
 * @return {void}
 */
function saveToLocalStorage<T>(key: string, value: T): void {
	try {
		const encoded = encodeToBase64(value);
		localStorage?.setItem(key, encoded);
	} catch {}
}

/**
 * Decodes a string value from Base64 encoding.
 *
 * @param {string} value - The Base64 encoded string to decode.
 * @template T - The type of the decoded value.
 * @returns {T} - The decoded value.
 */
function decodeFromBase64<T>(value: string) {
	const decoded = atob(value);
	return JSON.parse(decoded) as T;
}

/**
 * Encodes the given value to Base64.
 *
 * @param {T} value - The value to be encoded.
 * @returns {string} - The Base64 encoded string.
 */
function encodeToBase64<T>(value: T): string {
	const serializedState = JSON.stringify(value);
	return btoa(serializedState);
}
